import re
from collections import OrderedDict

from inventory_provider.db import ims
from inventory_provider.db.ims import IMS, InventoryStatus, RelateType

INTERNAL_POP_NAMES = {
    'Cambridge OC',
    'DANTE Lab',
    'GÉANT LAB',
    'GEANT LAB',
    'Amsterdam GEANT Office',
    'Amsterdam GÉANT Office'
}


def lookup_lg_routers(ds):
    pattern = re.compile("vpn-proxy|vrr|taas", re.IGNORECASE)

    def _matching_node(node_):
        if InventoryStatus(node_['InventoryStatusId']) not in [
            InventoryStatus.IN_SERVICE,
            InventoryStatus.PLANNED  # remove once data fully migrated
        ]:
            return False

        if pattern.match(node_['Name']):
            return False

        return True

    site_nav_props = [
        ims.SITE_PROPERTIES['aliases'],
        ims.SITE_PROPERTIES['city'],
        ims.SITE_PROPERTIES['country']
    ]

    eq_definitions = ds.get_filtered_entities(
        'EquipmentDefinition',
        'Name like mx',
        ims.EQUIP_DEF_PROPERTIES['nodes'])

    for eq_def in eq_definitions:
        nodes = eq_def['Nodes']

        for node in nodes:
            if not _matching_node(node):
                continue
            site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props,
                                       True)
            city = site['City']

            abbreviation = ''
            try:
                abbreviation = site['SiteAliases'][0]['AliasName']
            except IndexError:
                pass  # no alias - ignore silently

            eq = {
                    'equipment name': node['Name'],
                    'type':
                        'INTERNAL'
                        if site['Name'] in INTERNAL_POP_NAMES
                        else 'CORE',
                    'pop': {
                        'name': site['Name'],
                        'city': city['Name'],
                        'country': city['Country']['Name'],
                        'country code': city['Country']['Abbreviation'],
                        'abbreviation': abbreviation,
                        'longitude': site['Longitude'],
                        'latitude': site['Latitude'],
                    }
                }
            yield(eq)


def lookup_pop_info(ds, hostname):
    site_nav_props = [
        ims.SITE_PROPERTIES['aliases'],
        ims.SITE_PROPERTIES['city'],
        ims.SITE_PROPERTIES['country']
    ]

    node = ds.get_entity_by_name('Node', hostname)
    if not node:
        return None
    site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props, True)
    city = site['City']
    abbreviation = ''
    try:
        abbreviation = site['SiteAliases'][0]['AliasName']
    except IndexError:
        pass  # no alias - ignore silently
    eq = {
            'equipment-name': node['Name'],
            'status': InventoryStatus(node['InventoryStatusId']).name,
            'pop': {
                'name': site['Name'],
                'city': city['Name'],
                'country': city['Country']['Name'],
                'abbreviation': abbreviation,
                'longitude': site['Longitude'],
                'latitude': site['Latitude'],
            }
        }
    return eq


def get_interface_info(ds):
    yield from ds.get_all_entities('InternalPort')
    yield from ds.get_all_entities('Port')


def otrs_get_customer_company_rows(ds):
    yield ['customer_id', 'name', 'street', 'zip', 'city', 'country', 'url',
           'comments']
    for customer in ds.get_all_entities('Customer'):
        yield [customer['Name'], customer['FullName'], '', '', '', '', '', '']


def otrs_get_customer_users_rows(ds):
    yield ['email', 'username', 'customer_id', 'customer_id_2', 'title',
           'firstname', 'lastname', 'phone', 'fax', 'mobile', 'street', 'zip',
           'city', 'country', 'comments']
    type_priority = {
        RelateType.SITE: 1,
        RelateType.GROUP: 2,
        RelateType.CONTRACT: 3,
        RelateType.VENDOR: 4,
        RelateType.CONTACT: 5,
        RelateType.CUSTOMER: 6,
    }

    # TODO - check the rules for prioritisation once model has been confirmed
    def _choose_customer_user(cu_1, cu_2):
        if not cu_2 or type_priority.get(cu_1['type'], 0) > type_priority.get(
                cu_2['type'], 0):
            return cu_1
        return cu_2

    # TODO - check the rules for validation once model has been confirmed
    def _is_valid_customer(cus):
        # commented out as no contact details are in IMS
        # if not cus['email']:
        #     return False
        return True

    # TODO - check this rule for getting the customer_id2 once model has been confirmed  # noqa
    def _get_customer_id2(cus):
        if cus['CustomerType'] and cus['CustomerType']['Name'] == 'NREN NOC':
            return True
        return ''

    for customer in ds.get_all_entities(
            'Customer',
            [
                ims.CUSTOMER_PROPERTIES['CustomerRelatedContacts'],
                ims.CUSTOMER_PROPERTIES['CustomerType']
            ]):
        # Org as User
        org_customer = OrderedDict({
            'type': customer['CustomerType']['Name'] if customer[
                'CustomerType'] else '',
            'email': customer['AccountEmail'],
            'username': customer['AccountEmail'],
            'customer_id': customer['Name'],
            'customer_id_2': _get_customer_id2(customer),
            'title': '',
            'firstname': customer['Name'],
            'lastname': '-',
            'phone': '',
            'fax': '',
            'mobile': '',
            'street': '',
            'zip': '',
            'city': '',
            'country': '',
            'comments': ''
        })
        if _is_valid_customer(org_customer):
            del org_customer['type']
            yield org_customer.values()
        # end of Org as User

        # Contacts as Users
        if customer['CustomerRelatedContacts']:

            customer_user = None
            for contact in customer['CustomerRelatedContacts']:
                # could just do this as an array, but this makes it easier to
                # keep track of the fields
                t_customer_user = OrderedDict({
                    'type': ims.RelateType(contact['MainTypeId']),
                    'email': contact['Contact']['Mail'],
                    'username': contact['Contact']['Mail'],  # TODO if tal_id is going to be present use that # noqa
                    'customer_id': customer['Name'],
                    'customer_id_2': _get_customer_id2(customer),
                    'title': '',
                    'firstname': contact['Contact']['Name'],
                    'lastname': contact['Contact']['LastName'],
                    'phone': '',
                    'fax': '',
                    'mobile': '',
                    'street': '',
                    'zip': '',
                    'city': '',
                    'country': '',
                    'comments': ''
                })
                if not _is_valid_customer(t_customer_user):
                    continue
                customer_user = _choose_customer_user(
                    t_customer_user,
                    customer_user
                )
            if customer_user:
                del customer_user['type']
                yield customer_user.values()
        # end of Contact as User


if __name__ == '__main__':
    import sys
    username = sys.argv[1]
    password = sys.argv[2]
    ds = IMS('http://83.97.94.128:81/api', username, password)

    equipment_names = [
        'mx1.ams.nl',
        'mx1.ath2.gr',
        'mx1.buc.ro',
        'mx1.bud.hu',
        'mx1.dub.ie',
        'mx1.dub2.ie',
        'mx1.fra.de',
        'mx1.gen.ch',
        'mx1.ham.de',
        'mx1.kau.lt',
        'mx1.lis.pt',
        'mx1.lon.uk',
        'mx1.lon2.uk',
        'mx1.mad.es',
        'mx1.mar.fr',
        'mx1.mil2.it',
        'mx1.par.fr',
        'mx1.poz.pl',
        'mx1.pra.cz',
        'mx1.sof.bg',
        'mx1.tal.ee',
        'mx1.vie.at',
        'mx2.ath.gr',
        'mx2.bra.sk',
        'mx2.bru.be',
        'mx2.kau.lt',
        'mx2.lis.pt',
        'mx2.lju.si',
        'mx2.rig.lv',
        'mx2.tal.ee',
        'mx2.zag.hr',
        'qfx.fra.de',
        'qfx.lon2.uk',
        'qfx.par.fr'
    ]
    for equipment in equipment_names:
        r = lookup_pop_info(ds, equipment)
        print(r)

    print('--------------------')
    print('--------------------')

    for r in lookup_lg_routers(ds):
        print(r)
